/*
 * Copyright (C) 2025. Nyabsi <nyabsi@sovellus.cc>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package cc.sovellus.vrcaa.ui.screen.profile

import cafe.adriel.voyager.core.model.StateScreenModel
import cafe.adriel.voyager.core.model.screenModelScope
import cc.sovellus.vrcaa.App
import cc.sovellus.vrcaa.R
import cc.sovellus.vrcaa.api.search.avtrdb.AvtrDbProvider
import cc.sovellus.vrcaa.api.search.justhparty.JustHPartyProvider
import cc.sovellus.vrcaa.api.vrchat.http.interfaces.IFavorites
import cc.sovellus.vrcaa.api.vrchat.http.models.Instance
import cc.sovellus.vrcaa.api.vrchat.http.models.LimitedUser
import cc.sovellus.vrcaa.helper.ApiHelper
import cc.sovellus.vrcaa.manager.ApiManager.api
import cc.sovellus.vrcaa.manager.FavoriteManager
import kotlinx.coroutines.launch

class UserProfileScreenModel(
    private val userId: String
) : StateScreenModel<UserProfileScreenModel.UserProfileState>(UserProfileState.Init) {

    sealed class UserProfileState {
        data object Init : UserProfileState()
        data object Loading : UserProfileState()
        data object Failure : UserProfileState()
        data class Result(val profile: LimitedUser?, val instance: Instance?) : UserProfileState()
    }

    private val avatarProvider = AvtrDbProvider()

    private var profile: LimitedUser? = null
    private var instance: Instance? = null

    init {
        mutableState.value = UserProfileState.Loading
        fetchProfile()
    }

    private fun fetchProfile() {
        screenModelScope.launch {
            App.setLoadingText(R.string.loading_text_user)
            api.users.fetchUserByUserId(userId)?.let {
                it.location.let { location ->
                    if (it.isFriend &&
                        location.isNotEmpty() &&
                        location != "private" &&
                        location != "traveling" &&
                        location != "offline") {
                        instance = api.instances.fetchInstance(location)
                    }
                }
                profile = it
                mutableState.value = UserProfileState.Result(profile, instance)
            } ?: run {
                mutableState.value = UserProfileState.Failure
            }
        }
    }

    fun findAvatar(callback: ((userId: String?) -> Unit?)) {
        screenModelScope.launch {
            profile?.let {
                val fileId = ApiHelper.extractFileIdFromUrl(it.currentAvatarImageUrl)

                if (fileId != null) {
                    api.files.fetchMetadataByFileId(fileId)?.let { metadata ->

                        val name = metadata.name.split(" - ")

                        val (_, nameAvatars) = avatarProvider.search(name[1])
                        if (nameAvatars.isNotEmpty()) {
                            for (avatar in nameAvatars) {
                                avatar.imageUrl?.let {
                                    val avatarFileId = ApiHelper.extractFileIdFromUrl(avatar.imageUrl)
                                    if (avatarFileId == fileId) {
                                        callback(avatar.id)
                                        return@launch
                                    }
                                }
                            }
                        }

                        // fallback to using author search
                        val (_, authorAvatars) = avatarProvider.search(metadata.ownerId)
                        if (authorAvatars.isNotEmpty()) {
                            for (avatar in authorAvatars) {
                                avatar.imageUrl?.let {
                                    val avatarFileId = ApiHelper.extractFileIdFromUrl(avatar.imageUrl)
                                    if (avatarFileId == fileId) {
                                        callback(avatar.id)
                                        return@launch
                                    }
                                }
                            }
                        }
                        callback(null)
                    }
                }
            }
        }
    }

    fun inviteToFriend(intent: String) {
        screenModelScope.launch {
            api.instances.selfInvite(intent)
        }
    }

    fun removeFavorite(callback: (result: Boolean) -> Unit) {
        screenModelScope.launch {
            profile?.let {
                val result = FavoriteManager.removeFavorite(IFavorites.FavoriteType.FAVORITE_FRIEND, it.id)
                callback(result)
            }
        }
    }
}